@@ -30,30 +30,43 @@ from tabulate import simple_separated_format
 
 docstrings = {
     'input_ts': '''input_ts : str
+        Whether from a file or standard input, data requires a first line
+        header of column names.  Most separators will be automatically
+        detected. Most common date formats can be used, but the closer to ISO
+        8601 date/time standard the better.
+
         Command line:
 
-            --input_ts= file name or '-' for standard input (stdin).
-            Default is stdin.
+            `--input_ts=filename.csv` or '--input_ts=-' for standard input
+            (stdin).  Default is '-'.
 
-            If stdin:
+            In many cases it is better to use redirection rather that use
+            `--input_ts=filename.csv`.  The following are identical:
 
-                The data must be formatted as "," separated data, with
-                a first line header for column names.
+            From a file:
 
-            If file name:
+                command subcmd --input_ts=filename.csv
 
-                Will try to figure out separator, but still requires
-                a first line header of column names.
+            From standard input:
+
+                command subcmd --input_ts=- < filename.csv
+
+            The BEST way since you don't have to include `--input_ts=-` because
+            that is the default:
+
+                command subcmd < filename.csv
+
+            Can also combine commands by piping:
+
+                command subcmd < filename.csv | command subcmd1 > fileout.csv
 
         As Python Library:
 
-            `input_ts=` {pandas Dataframe, pandas Series, dict, tuple,
-            list, StringIO, file name}
+            You MUST use the `input_ts=...` option where `input_ts` can be one
+            of a [pandas Dataframe, pandas Series, dict, tuple,
+            list, StringIO, or file name].
 
             If result is a time series, returns a pandas DataFrame.
-
-        Most common date formats can be used, but the closer to ISO 8601
-        date/time standard the better.
         ''',
     'columns': '''columns
         Columns to select out of input.  Can use column names from the
@@ -94,8 +107,7 @@ docstrings = {
 
 
 def b(s):
-    """Make sure all strings are correctly represented in Python 2 and
-    3.."""
+    """Make sure strings are correctly represented in Python 2 and 3."""
     try:
         return s.encode("utf-8")
     except AttributeError:
@@ -120,12 +132,14 @@ def parsedate(dstr,
     import dateparser
     import datetime
 
-    # This is here just to make a generic tool.
-    if dstr is None:
-        return None
-
-    # The API should boomerang a datetime.datetime instance.
+    # The API should boomerang a datetime.datetime instance and None.
     if isinstance(dstr, datetime.datetime):
+        if strftime is None:
+            return dstr
+        else:
+            return dstr.strftime(strftime)
+
+    if dstr is None:
         return dstr
 
     pdate = dateparser.parse(dstr, settings=settings)
@@ -267,7 +281,7 @@ def _pick(tsd, columns):
             # if using column numbers
             try:
                 target_col = int(i) - 1
-            except:
+            except ValueError:
                 raise ValueError("""
 *
 *   The name {0} isn't in the list of column names
@@ -277,7 +291,7 @@ def _pick(tsd, columns):
             if target_col < 0:
                 raise ValueError("""
 *
-*   The request column index {0} must be greater than or equal to 0.
+*   The requested column index {0} must be greater than or equal to 0.
 *   First data column is index 1, index is column 0.
 *
 """.format(i))
@@ -310,16 +324,6 @@ def _pick(tsd, columns):
     return newtsd
 
 
-def date_slice(input_tsd, start_date=None, end_date=None):
-    """Limit input to start_date and end_date.
-
-    This is here for a while until I fix my other toolboxes to
-    use common_kwds instead.
-    """
-    return _date_slice(input_tsd,
-                       start_date=start_date,
-                       end_date=end_date)
-
 
 def _date_slice(input_tsd,
                 start_date=None,
@@ -390,6 +394,24 @@ def asbestfreq(data, force_freq=None):
     if not isinstance(data.index, pd.DatetimeIndex):
         return data
 
+    ndiff = (data.index.values.astype('int64')[1:] -
+             data.index.values.astype('int64')[:-1])
+    if np.any(ndiff == 0):
+        raise ValueError("""
+*
+*   Duplicate index entry record {1} (start count at 0):
+*   "{0}".
+*
+""".format(data.index[:-1][ndiff == 0][0], pd.np.where(ndiff == 0)[0][0] + 1))
+    if np.any(ndiff < 0):
+        raise ValueError("""
+*
+*   Time warp.  The datetime index is not sorted at
+*   record {1} (start count at 0):
+*   "{0}".
+*
+""".format(data.index[ndiff < 0][0], pd.np.where(ndiff < 0)[0][0] + 1))
+
     if force_freq is not None:
         return data.asfreq(force_freq)
 
@@ -444,8 +466,7 @@ def asbestfreq(data, force_freq=None):
 
     # Use the minimum of the intervals to test a new interval.
     # Should work for fixed intervals.
-    ndiff = sorted(set(data.index.values.astype('int64')[1:] -
-                       data.index.values.astype('int64')[:-1]))
+    ndiff = sorted(set(ndiff))
     mininterval = np.min(ndiff)
     if mininterval <= 0:
         raise ValueError
@@ -567,6 +588,9 @@ def _printiso(tsd,
     elif isinstance(tsd, (int, float, list, tuple, pd.np.ndarray)):
         tablefmt = None
 
+    tsd.rename(lambda x: x.replace(',', '_').replace(' ', '_').replace('__', '_'),
+               axis='columns',
+               inplace=True)
     if tablefmt in ["csv", "tsv", "csv_nos", "tsv_nos"]:
         sep = {"csv": ",",
                "tsv": "\\t",
@@ -750,13 +774,18 @@ def read_iso_ts(indat,
             sep = None
             fpi = indat
             fname = ''
+        else:
+            raise ValueError("""
+*
+*   Can't figure out what "input_ts={0}" is.
+*   I tested if it was a string or StringIO object, DataFrame, local file,
+*   or an URL.  If you want to pull from stdin use "-".
+*
+""".format(indat))
 
         fstr = '{1}'
         if extended_columns is True:
-            try:
-                fstr = '{0}.{1}'
-            except:
-                pass
+            fstr = '{0}.{1}'
 
         index_col = 0
         if parse_dates is False:
